/**
 * @file
 * lazy_image_style behaviors.
 */

(function (Drupal, DrupalSettings, debounce) {

  'use strict';


  const lazyLoadHandler = {

    /**
     * Selector for find lazy images.
     */
    findSelector: 'img[data-src]',

    /**
     * Init LazyLoadHandler.
     */
    init: debounce(function () {
      let lazyImages = lazyLoadHandler.getObjects();
      lazyLoadHandler.loadObjects(lazyImages);
    }, 300, true),

    /**
     * Return variable from drupal settings.
     *
     * @param key
     * @param default_value
     * @returns {*}
     */
    getConfig: function (key, default_value) {
      let lazy_image_style = DrupalSettings && DrupalSettings.hasOwnProperty('lazy_image_style');

      if (lazy_image_style) {
        return lazy_image_style.hasOwnProperty(key) ?
            DrupalSettings.lazy_image_style[key] : default_value;
      }

      return default_value;
    },

    /**
     * Return all images supported lazy load.
     *
     * @returns {...NodeListOf<Element>[]}
     */
    getObjects: function () {
      return [...document.querySelectorAll(this.findSelector)];
    },

    /**
     * Load objects.
     *
     * @param {...NodeListOf<Element>[]} elems
     */
    loadObjects: function (elems) {
      let offset = Number(this.getConfig('offset', 100));
      let type = this.getConfig('type', 'tmp_img');

      elems.forEach(image => {
        if ((image.getBoundingClientRect().top <= window.innerHeight + offset
            && image.getBoundingClientRect().bottom >= 0) && getComputedStyle(image).display !== 'none') {
          image.src = image.dataset.src;

          if (image.classList.contains('blue')) {
            image.classList.remove('blue');
          }

          elems = elems.filter(function (elem) {
            return elem !== image;
          });
          if (elems.length === 0) {
            document.removeEventListener('scroll', lazyLoadHandler.init);
            window.removeEventListener('resize', lazyLoadHandler.init);
            window.removeEventListener('orientationchange', lazyLoadHandler.init);
          }
        }
      });
    }
  };

  /**
   * Find lazy img and set src from data-src.
   */
  Drupal.behaviors.lazyImageStyle = {
    attach: function (context, settings) {
      lazyLoadHandler.init();
      document.addEventListener('scroll', lazyLoadHandler.init);
      window.addEventListener('resize', lazyLoadHandler.init);
      window.addEventListener('orientationchange', lazyLoadHandler.init);
    }
  };

}(Drupal, drupalSettings, Drupal.debounce));
